package com.annimon.ownlang.errorhighlighting;

import com.annimon.ownlang.parser.OwnLangParserResult;
import com.annimon.ownlang.parser.ParseError;
import com.annimon.ownlang.parser.Parser;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import javax.swing.text.Document;
import org.netbeans.modules.parsing.spi.ParseException;
import org.netbeans.modules.parsing.spi.Parser.Result;
import org.netbeans.modules.parsing.spi.ParserResultTask;
import org.netbeans.modules.parsing.spi.Scheduler;
import org.netbeans.modules.parsing.spi.SchedulerEvent;
import org.netbeans.spi.editor.hints.ErrorDescription;
import org.netbeans.spi.editor.hints.ErrorDescriptionFactory;
import org.netbeans.spi.editor.hints.HintsController;
import org.netbeans.spi.editor.hints.Severity;
import org.openide.util.Exceptions;

public final class OwnLangSyntaxErrorHighlightingTask extends ParserResultTask {

    @Override
    public void run(Result result, SchedulerEvent event) {
        try {
            final OwnLangParserResult olResult = (OwnLangParserResult) result;
            final Parser parser = olResult.getParser();
            if (!parser.getParseErrors().hasErrors()) {
                final Document document = result.getSnapshot().getSource().getDocument(false);
                HintsController.setErrors(document, "ownlang", new ArrayList<ErrorDescription>());
                return;
            }

            final Document document = result.getSnapshot().getSource().getDocument(false);
            final List<ErrorDescription> errors = new ArrayList<ErrorDescription>();
            for (ParseError parseError : parser.getParseErrors()) {
                ErrorDescription error = ErrorDescriptionFactory.createErrorDescription(
                        Severity.ERROR, parseError.getException().getMessage(),
                        Collections.EMPTY_LIST, document, parseError.getLine());
                errors.add(error);
            }
            HintsController.setErrors(document, "ownlang", errors);
        } catch (ParseException ex) {
            Exceptions.printStackTrace(ex);
        }
    }

    @Override
    public int getPriority() {
        return 100;
    }

    @Override
    public Class<? extends Scheduler> getSchedulerClass() {
        return Scheduler.EDITOR_SENSITIVE_TASK_SCHEDULER;
    }

    @Override
    public void cancel() {
    }
}
