package com.annimon.ownlang.parser;

import com.annimon.ownlang.lexer.Lexer;
import com.annimon.ownlang.parser.visitors.IdentifiersVisitor;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Set;
import javax.swing.text.Document;
import javax.swing.text.JTextComponent;
import org.netbeans.modules.csl.api.*;
import org.netbeans.modules.csl.spi.DefaultCompletionResult;
import org.netbeans.modules.csl.spi.ParserResult;
import org.openide.util.Exceptions;

public final class OwnLangCompletionHandler implements CodeCompletionHandler {

    @Override
    public CodeCompletionResult complete(CodeCompletionContext ccc) {
        final List<CompletionProposal> proposals = new ArrayList<CompletionProposal>();
        // Identifiers
        try {
            OwnLangParserResult olResult = (OwnLangParserResult) ccc.getParserResult();
            final Parser parser = olResult.getParser();
            if (!parser.getParseErrors().hasErrors() && parser.getParsedStatement() != null) {
                IdentifiersVisitor iv = new IdentifiersVisitor();
                Set<Identifier> identifiers = iv.execute(parser.getParsedStatement());
                for (Identifier identifier : identifiers) {
                    OwnLangElementHandle element = OwnLangElementHandle
                            .with(identifier.getName(), new OffsetRange(2, 10), identifier.getKind())
                            .build();
                    proposals.add(new OwnLangCompletionProposal(element));
                }
            }
        } catch (Exception ex) {
            Exception wrap = new Exception(ccc.getParserResult().getClass().getName(), ex);
            Exceptions.printStackTrace(wrap);
        }
        // Keywords
        for (String keyword : Lexer.getKeywords().keySet()) {
            OwnLangElementHandle element = OwnLangElementHandle
                    .with(keyword, new OffsetRange(2, 10), ElementKind.KEYWORD)
                    .build();
            proposals.add(new OwnLangCompletionProposal(element));
        }
        return new DefaultCompletionResult(proposals, false);
    }

    @Override
    public String document(ParserResult pr, ElementHandle eh) {
        return "";
    }

    @Override
    public ElementHandle resolveLink(String string, ElementHandle eh) {
        return null;
    }

    @Override
    public String getPrefix(ParserResult info, int caretOffset, boolean upToOffset) {
        return "";
    }

    @Override
    public QueryType getAutoQuery(JTextComponent jtc, String string) {
        return QueryType.ALL_COMPLETION;
    }

    @Override
    public String resolveTemplateVariable(String string, ParserResult pr, int i, String string1, Map map) {
        return null;
    }

    @Override
    public Set<String> getApplicableTemplates(Document dcmnt, int i, int i1) {
        return Collections.emptySet();
    }

    @Override
    public ParameterInfo parameters(ParserResult pr, int i, CompletionProposal cp) {
        return new ParameterInfo(new ArrayList<String>(), 0, 0);
    }

}
