package com.annimon.ownlang.parser.visitors;

import com.annimon.ownlang.parser.Identifier;
import com.annimon.ownlang.parser.ast.*;
import java.util.HashSet;
import java.util.Set;
import org.netbeans.modules.csl.api.ElementKind;

/**
 *
 * @author aNNiMON
 */
public final class IdentifiersVisitor extends AbstractVisitor {
    
    private final Set<Identifier> identifiers;

    public IdentifiersVisitor() {
        identifiers = new HashSet<>();
    }
    
    public Set<Identifier> execute(Statement statement) {
        identifiers.clear();
        statement.accept(this);
        return identifiers;
    }
    
    @Override
    public void visit(ContainerAccessExpression s) {
        super.visit(s);
        identifiers.add(new Identifier(s.variable, ElementKind.VARIABLE));
    }

    @Override
    public void visit(VariableExpression s) {
        super.visit(s);
        identifiers.add(new Identifier(s.name, ElementKind.VARIABLE));
    }

    @Override
    public void visit(DestructuringAssignmentStatement s) {
        super.visit(s);
        for (String variable : s.variables) {
            identifiers.add(new Identifier(variable, ElementKind.VARIABLE));
        }
    }

    @Override
    public void visit(FunctionDefineStatement s) {
        super.visit(s);
        identifiers.add(new Identifier(s.name, ElementKind.METHOD));
        /*for (Argument argument : s.arguments) {
            if (argument.getName() != null) {
                identifiers.add(new Identifier(argument.getName(), ElementKind.PARAMETER));
            }
        }*/
    }
}
